/**
 * File Name: Mainp3.js
 * Live Demo: https://makingbrowsergames.com/p3gp-book/v3.x.x/
 * Source Code: https://makingbrowsergames.com/p3gp-book/
 * Description: File for controlling and displaying game scenes;
 *               managing global variables throughout game state.

 * Author: Stephen Gose
 * Version: 0.0.0.20180805-1  //semantic version: by (date-#)
 * Phaser Version: 3.x.x.
 * Author URL: https://www.stephen-gose.com/
 * Support: support@pbmcube.com
 *
 * \u00A9 Copyright ©  1974-2017 Stephen Gose LLC. All rights reserved. 
 * Redistribution of part or whole of this file and
 *   the accompanying files is strictly prohibited.
 * 
 * Do not sell! Do not distribute!  
 *  Please refer to the Terms of Use and 
 *    End Users License Agreement (EULA).
 *  This is a licensed file with EULA permission. 
 *  
 * Search for "//**TODO**" to tailor this file for your production; 
 *	 doing so will void any support agreement.
 *
 * Construction References:
 * https://www.adequatelygood.com/JavaScript-Module-Pattern-In-Depth.html
 *
 */
 
"use strict";
// =========================================================
/**	game set-up	
 *  Beginning of Static internal game database(s)
 *  //**TODO** 
 *  NOTE: the following data structures are for development only.
 *  The optimum usage is a local or remote database using PouchDB or SQLite.
 *  Stay away from IndexDB since it is deprecated.
 * 
 * -------------------------------------------
 * Anti-Pattern: global name-space pollution by
 *   Standard Phaser v3.x.x Scene preparations
 * init:     function init() {},    //initial game phase data  
 * preload:  function preload() {}, //queue & download game assets 
 * create:   function create() {},  //make cached assets available 
 * update:   function update() {},  //begin the game loop  
 * render:   function render() {},  //render current display 
 * shutdown: function shutdown() {} //close and garbage collection? 
 * -------------------------------------------
 */

// -------------------------------------------
// Acknowledgments:
// =========================================================
console.log ("%c %c %c|   Starting my awesome MMoG game Prototype!       |%c %c ","background: #00bff3","background: #0072bc","color: #ffffff; background: #003471","background: #0072bc","background: #00bff3");
console.log ("%c %c %c|     Phaser 3 Game Prototyping Starter Kit        |%c %c ","background: #00bff3","background: #0072bc","color: #ffffff; background: #003471","background: #0072bc","background: #00bff3");
console.log ("%c %c %c|     Copyright \u00A9 1974-2017, Stephen Gose.         |%c %c ","background: #00bff3","background: #0072bc","color: #ffffff; background: #003471","background: #0072bc","background: #00bff3");
console.log ("%c %c %c| \u2665\u2665\u2665\u2665\u2665 -> https://shop.pbmcube.net/                |%c %c ","background: #00bff3","background: #0072bc","color: #ffffff; background: #003471","background: #0072bc","background: #00bff3");
console.log ("%c %c %c| \u2665\u2665\u2665\u2665\u2665 -> $120 License included in book!          |%c %c ","background: #00bff3","background: #0072bc","color: #ffffff; background: #003471","background: #0072bc","background: #00bff3");
console.log ("%c %c %c| Book: https://leanpub.com/phaser3gameprototyping |%c %c ","background: #00bff3","background: #0072bc","color: #ffffff; background: #003471","background: #0072bc","background: #00bff3");

var boardGroup;
var selectedTile;

// creates a game name-space
// =========================================================
//**TODO** Creating Game name-space called "GAMEAPP" object;
// rename "GAMEAPP" with your project's name
var GAMEAPP = {
	//US Copr. or Copyright; UTF8 "circled c" is \u00A9 equivalent to &copy;
	Copr: "\u00A9 Copyright © 1974-2016, Stephen Gose. All rights reserved.\n",
	
	//assign a unique identification (SKU code) for local storage
	gDefDelim: "game SKU",	//See cookie/local.storage below
	
	// If there's music in your game, and it needs to play through-out a 
	//	  few State swaps, then you could reference it below. 
	//music: null,
	
	// Toggle background music theme on or off; starts in "on/true" state 
	//musicToggle: true,
	
	// Your game can check MYGAMEAPP.orientated in the game loops 
	//	  to know if it should pause or not.
	//orientated: false,
	
	//Canvas dimensions: world and viewport Height and Width
	//**TODO** adjust for your game deployment
	viewportWidth:	800,	//game view using Golden Ration
	viewportHeight:	500,
	worldWidth:		800,	//world view using Golden Ration
	worldHeight:	500,
	
	//==================================================
	// -------------------------------------------
	//	Trash Global Variables (alphabetical)
	// -------------------------------------------
	//**TODO** 
	//  Refactor and adjust for your own game deployment
	//	Here we have some global level vars that persist regardless of State. 
	//==================================================
	gameLanguage: 0,	//Default is English (American)
	InfoText: "Default Text.",	
	//button styling
	styleBTN: { font: "26px Arial", fill: "#000000", align: "center" },
	
	//Combat HUD styling
	//	hero text
	styleCmbtH: { font: "9px Arial", fill: "#66ffff", align: "center" },	
	styleNarrH: { font: "9px Arial", fill: "#66ffff", align: "left" },
	//	monster text
	styleCmbtM: { font: "9px Arial", fill: "#66ff66", align: "center" },
	styleNarrM: { font: "9px Arial", fill: "#66ff66", align: "left" },
	
	//HUD styling
	styleHUD: { font: "14px Arial", fill: "#ff9900", align: "center" },
	//Room Alert styling
	styleRA: { font: "18px Arial", fill: "#ff9900", align: "center" },
	//Dynamic Story style
	styleStory: { font: "16px Arial", fill: "#ff9900", align: "left" },
	//toolTip styling
	styleTT: { font: "11px Arial", fill: "#ff9900", align: "center" },
	
	// =========================================================
	// -------------------------------------------
	// Main game Handler methods
	// -------------------------------------------
	//**TODO**
	//	re-factor and adjust for your game deployment
	//	remove console debug information BEFORE public deployment
	// =========================================================
	main: function main(){
		
		this.game = new Phaser.Game(config);
		
		// add all game phases into Phaser v3.x.x scenes.
		for(var stateName in window.GAMEAPP.state){
			console.log("Creating Crnt State: "+stateName);
			this.game.scene.add(stateName, window.GAMEAPP.state[stateName]);
			
		}
	
		//using v3? use this manual start method below.
		console.log("Leaving GAMEAPP.main -> boot");	//debug
		this.game.scene.start('boot');
	},
	// =========================================================
	/**
	 *  NEW Phaser v3.x.x method: passing via Object.create
	 *      configuration submitted as external object
	 *  
	 *   this.game = Object.create(Phaser);
	 *  
	 *  Phaser docs suggests a separate object literal config = {...}; 
	 *  See "Phaser 3 Game Prototyping" book reference  
	 *    for all the game configuration parameters. 
	 *  Available from 
	 *    https://leanpub.com/phaser3gameprototyping or search Amazon.com 
	 *  
	 * Experiment: Embed Config into Phaser v3.x.x directly. 
	 *  Note we could have simply inserted the object directly.
	 *  You'll occasionally find this methods in the Phaser3 lab examples.
	 *  https://labs.phaser.io/
	 *  

		this.game = new Phaser.Game(
			//internalized configuration object
			// One less global pollution!
			{
				width: window.GAMEAPP.viewportWidth,
				height: window.GAMEAPP.viewportHeight,
				type: Phaser.AUTO,
				scene: window.GAMEAPP.state,
				parent: gameCanvas	//or document.body
				
			});
	*/
	
	// =========================================================
	// here we will store all game phase/states
	//  state object filled as js/ files load.
	state: {},
	
	// =================================================
	// -------------------------------------------
	// Supporting game Function & Classes
	// -------------------------------------------
	//**TODO**:
	//	Change name-space from generic GAMEAPP to your project 
	//	refactor and adjust for your game deployment
	//	remove console debug information on public deployment
	// 
	// =================================================
	//This could be loaded externally from a JSON file
	//I choose to have this internalized to avoid additional downloads.
	// When loading a file from a different domain, the game must have 
	//  a 'Access-Control-Allow-Origin' header present.
	_assignLanguage: function(){
		
		switch(GAMEAPP.gameLanguage){
			case 0:	//"english":
				console.log("English selected");
				GAMEAPP.credittxt= "Credits & Info";
				GAMEAPP.donatxt= "Support Us";
				GAMEAPP.helptxt= "Help & Instructions";
				GAMEAPP.introPara= "Introduction game-text here.\n";
				GAMEAPP.moretxt= "More games";
				GAMEAPP.opttxt= "Options";
				GAMEAPP.playtxt= "Start Game";
				GAMEAPP.sharetxt= "Share with friends";
				GAMEAPP.webtxt= "Webmaster License Info";
				break;
			
			case 1: //"china"
				GAMEAPP.credittxt = "片尾";
				GAMEAPP.donatxt = "";
				GAMEAPP.helptxt = "游戏说明";
				GAMEAPP.introPara = "";
				GAMEAPP.moretxt = "更多的游戏";
				GAMEAPP.opttxt = "游戏选项";
				GAMEAPP.playtxt = "玩这个游戏";
				GAMEAPP.sharetxt = "分享此游戏";
				GAMEAPP.webtxt =  "";
				break;
			
			case 2:	//"denmark":
				GAMEAPP.credittxt = "afsluttende credits";
				GAMEAPP.donatxt =  "";
				GAMEAPP.helptxt = "spil instruktioner";
				GAMEAPP.introPara = "";
				GAMEAPP.moretxt = "flere spil";
				GAMEAPP.opttxt = "spilindstillinger";
				GAMEAPP.playtxt = "spille dette spil";
				GAMEAPP.sharetxt = "deler dette spil";
				GAMEAPP.webtxt =  "";	
				break;
			
			case 3:	//"french":
				GAMEAPP.credittxt = "générique de fin";
				GAMEAPP.donatxt =  "";
				GAMEAPP.helptxt = "Instructions du jeu";
				GAMEAPP.introPara = "";
				GAMEAPP.moretxt = "plus de jeux de";
				GAMEAPP.opttxt = "options de jeu";
				GAMEAPP.playtxt = "Jouer à ce jeu";
				GAMEAPP.sharetxt = "partager ce jeu";
				GAMEAPP.webtxt =  "";
				break;

			case 4:	//"german":
				GAMEAPP.credittxt = "Abspann";
				GAMEAPP.donatxt =  "";
				GAMEAPP.helptxt = "Anweisungen";
				GAMEAPP.introPara =""; 
				GAMEAPP.moretxt = "mehr";
				GAMEAPP.opttxt = "Spieloptionen";
				GAMEAPP.playtxt = "Spiel";
				GAMEAPP.sharetxt = "Aktien dieser";
				GAMEAPP.webtxt =  "";
				break;	
				
			case 5:	//"italian":
				GAMEAPP.credittxt = "Associare";
				GAMEAPP.donatxt =  "";
				GAMEAPP.helptxt = "Istruzioni di gioco";
				GAMEAPP.introPara = "";
				GAMEAPP.moretxt = "Quantità";
				GAMEAPP.opttxt = "Opzioni del gioco";
				GAMEAPP.playtxt = "Giocare questo gioco";
				GAMEAPP.sharetxt = "Condividere questo gioco";
				GAMEAPP.webtxt =  "";
				break;
			
			case 6:	//"japanese":
				GAMEAPP.credittxt = "閉じるクレジット";
				GAMEAPP.donatxt =  "";
				GAMEAPP.helptxt = "ゲームの説明";
				GAMEAPP.introPara = "";
				GAMEAPP.moretxt = "より多くのゲーム";
				GAMEAPP.opttxt = "ゲームオプション";
				GAMEAPP.playtxt = "このゲームをプレイ";
				GAMEAPP.sharetxt = "共有このゲーム";
				GAMEAPP.webtxt =  "";
				break;
			
			case 7:	//"netherlands":
				GAMEAPP.credittxt = "Credits";
				GAMEAPP.donatxt =  "";
				GAMEAPP.helptxt = "Spelregels";
				GAMEAPP.introPara =  "";
				GAMEAPP.moretxt = "meer games";
				GAMEAPP.opttxt = "spelopties";
				GAMEAPP.playtxt = "dit spel";
				GAMEAPP.sharetxt = "aandeel dit spel";
				GAMEAPP.webtxt =  "";
				break;
			
			case 8:	//"portugal":
				GAMEAPP.credittxt = "créditos finais";
				GAMEAPP.donatxt =  "";
				GAMEAPP.helptxt = "Instruções de jogo";
				GAMEAPP.introPara = "";
				GAMEAPP.moretxt = "mais jogos";
				GAMEAPP.opttxt = "opções de jogo";
				GAMEAPP.playtxt = "Jogar este jogo";
				GAMEAPP.sharetxt = "Compartilhe este jogo";
				GAMEAPP.webtxt =  "";
				break;
			
			case 9:	//"spain":
				GAMEAPP.credittxt = "Créditos";
				GAMEAPP.donatxt =  "";
				GAMEAPP.helptxt = "Instrucciones";
				GAMEAPP.introPara = "";
				GAMEAPP.moretxt = "Más";
				GAMEAPP.opttxt = "Opciones de juego";
				GAMEAPP.playtxt = "Jugar a este juego";
				GAMEAPP.sharetxt = "Compartir esta";
				GAMEAPP.webtxt = "";		
				break;
			
			case 10:	//"sweden":
				GAMEAPP.credittxt = "eftertexterna";
				GAMEAPP.donatxt =  "";
				GAMEAPP.helptxt = "instruktioner";
				GAMEAPP.introPara = "";
				GAMEAPP.moretxt = "mer";
				GAMEAPP.opttxt = "Spelalternativ";
				GAMEAPP.playtxt = "Spela";
				GAMEAPP.sharetxt = "VARA DELAKTIG I";
				GAMEAPP.webtxt =  "";
				break;
			
			default:
				console.log("English selected");
				GAMEAPP.credittxt= "Credits & Info";
				GAMEAPP.donatxt= "Support Us";
				GAMEAPP.helptxt= "Help & Instructions";
				GAMEAPP.introPara= "";
				GAMEAPP.moretxt= "More games";
				GAMEAPP.opttxt= "Options";
				GAMEAPP.playtxt= "Start Game";
				GAMEAPP.sharetxt= "Share with friends";
				GAMEAPP.webtxt= "Webmaster License Info";
				
				break;
		}
	},	

	// 
	// =================================================
	// -------------------------------------------
	// End Main game Handler
	// -------------------------------------------
	// =================================================
};

console.log("Game obj: === Ext? "+Object.isExtensible( GAMEAPP ));
//console.log(Object.values(GAMEAPP));
//console.log(Object.getPrototypeOf(GAMEAPP));
console.log(Object.getOwnPropertyDescriptors(GAMEAPP));

var config = {
	width:    window.GAMEAPP.viewportWidth,
	height:   window.GAMEAPP.viewportHeight,
	type:     Phaser.AUTO,
	title: 'Phaser3 Game Prototyping Starter Kit',		//Game Title
	url: 'https://makingbrowsergames.com/p3gp-book/',	//Game URL location
	version: '0.0.1.0 semver ',	//https://semver.org/spec/v1.0.0.html
	input: {
		keyboard: true,
		mouse: true,
		touch: true,
		gamepad: false
	},
	//scene:  window.GAMEAPP.state,
	//parent: document.body
	parent:   gameCanvas
};
/**
 * Standard global pollution in Phaser v3.x.x preparations
 * Configuration as an object literal: NOT HOISTED
//Using function method; JS Hoisted!
var config = function config() {
		var width = window.GAMEAPP.viewportWidth,
		height = window.GAMEAPP.viewportHeight,
		type = Phaser.AUTO,
		parent = gameCanvas;
	};
*/	
console.log("Configure Obj: === Ext? "+Object.isExtensible( config ));
//console.log(Object.values(config));	
//console.log(Object.getPrototypeOf(config));
console.log(Object.getOwnPropertyDescriptors(config));

/** 
 *  DEPRECATED METHOD - NEVER EVER USE THIS AGAIN!
 * 	See "Phaser III Game Design Workbook" for complete explanation
 *	https://leanpub.com/phaser3gamedesignworkbook
 *
 * 	window.onload = function () {
 *	    let game =  . . . . . .
 *	};
*/

//preferred game launch method.
document.addEventListener('DOMContentLoaded', function(){
	window.GAMEAPP.main();
}, false);

console.log("Game prototype (Phaser.Game): === Ext? "+Object.isExtensible( Phaser ));
//console.log(Object.values(Phaser));
//console.log(Object.getPrototypeOf(Phaser));
console.log(Object.getOwnPropertyDescriptors(Phaser));

//
// =================================================
/** End of file */
/** Location: ./js/main.js */
  